*=----------------------------------------------------------------------------
* Program...VARFILES.PRG                             Save & Restore variables
* Version...1.00  9101.29 / 12:56  TGM
* Notes.....VarSave(x2Save, cFile) - Saves a variable to cFile
*           VarLoad(@x2Load, cFile) - Loads a variable from cFile
*           
* Notice....Copyright (C) 1991 by Terry McConnell
*=----------------------------------------------------------------------------
#translate true => .T.
#translate false => .F.

*--------------------------------------------------  Write/Read Elem need these
#define TYPENIL   CHR(0)
#define TYPEChar  CHR(1)
#define TYPETrue  CHR(2)
#define TYPEFalse CHR(3)
#define TYPENum   CHR(4) 
#define TYPEDate  CHR(5)
#define TYPEArray CHR(6)

/*-------------------------------------------------  VarSave
* Syntax....VarSave(x2Save,cFile)
* Purpose...Saves 'x2Save' to a file specified by cFile
* Returns...0 if it worked, FERROR() if it didn't.
*           'cFile' will be obliterated.
* Version...1.00  9101.29 / 12:57
*/
FUNCTION VarSave(x2Save,cFile)
LOCAL nHan                                       //  File Handle

BEGIN SEQUENCE                                   //  Used to trap FERROR()s
	IF FILE(cFile)
		FerrChk(FERASE(cFile))
	ENDIF

	nHan := FerrChk(FCREATE(cFile))
	WriteElem(x2Save,nHan)

	FerrChk(FCLOSE(nHan))
END SEQUENCE
	
RETURN(FERROR())


/*-------------------------------------------------  WriteElem
* Syntax....WriteElem(xThing, nHan)
* Purpose...Writes the thing in xThing to an open file specified
*           by nHan, in a way that can be read by ReadElem()
* Returns...NIL
* Version...1.00  9101.29 / 13:09
* Notice....Copyright (C) 1991 by T. McConnell
*/
STATIC FUNCTION WriteElem(xThing, nHan)

LOCAL cOutString := ""                           //  String to send to file
LOCAL nLen       := 0                            //  (Avoid redundancy later)
LOCAL nDec       := 0                            //  (Avoid redundancy later)
LOCAL lRecurs    := false                        //  If true, AEVAL() begins after 
                                                 //  cOutstring is written ...
	DO CASE
		CASE VALTYPE(xThing) $  "CM"               //  Char or memo
			cOutString := TYPEChar + I2BIN(LEN(xThing)) + xThing

		CASE VALTYPE(xThing) == "L"                //  Logical
		   cOutString := IIF(xThing,TYPETrue,TYPEFalse)

		CASE VALTYPE(xThing) == "N"                //  Numeric
		   cOutString := TYPENum + CHR(nLen := StrLen(xThing)) +;
			              CHR(nDec := StrDec(xThing)) + STR(xThing,nLen+nDec+1,nDec)

		CASE VALTYPE(xThing) == "D"                //  Date
			cOutString := TYPEDate + DTOS(xThing)

		CASE VALTYPE(xThing) == "A"                //  Array
			cOutString := TYPEArray + I2BIN(LEN(xThing))
			lRecurs    := true

		CASE VALTYPE(xThing) == "U"                //  Undefined
			cOutString := TYPENIL

		CASE VALTYPE(xThing) == "B"                //  Block (Can't Save)
			cOutString := TYPENIL
			
		CASE VALTYPE(xThing) == "O"                //  Object (Can't Save)
			cOutString := TYPENIL
			
		OTHERWISE                                  //  Unknown - Can't Save
			cOutString := TYPENIL
	ENDCASE

	IF FerrChk(FWRITE(nHan,cOutString)) # LEN(cOutString)
		BREAK
	ENDIF

	IF lRecurs
		AEVAL(xThing,{|e|WriteElem(e,nHan)})       //  Recursion starts here
	ENDIF

RETURN(NIL)


/*-------------------------------------------------  StrLen
* Syntax....StrLen(nNum)
* Purpose...Calculates the integer length of nNum
* Returns...Integer length of nNum
* Version...1.00  9101.29 / 13:42
* Notice....Copyright (C) 1991 by T. McConnell
*/
STATIC FUNCTION StrLen(nNum)
LOCAL nRetVal := 0                               //  The retval

nNum := INT(nNum)                                

WHILE !(nNum == 0)
	nNum := INT(nNum/10)
	nRetVal++
END

RETURN(nRetVal)


/*-------------------------------------------------  StrDec
* Syntax....StrDec(nNum)
* Purpose...Calculates the string length of decimals in nNum
* Returns...Number of decimals in nNum
* Version...1.00  9101.29 / 13:51
* Notice....Copyright (C) 1991 by T. McConnell
*/
STATIC FUNCTION StrDec(nNum)
LOCAL cTemp   := STR(nNum)
LOCAL nRetVal := 0

nNum := VAL("0."+SUBSTR(cTemp,AT(".",cTemp)+1))

WHILE !(nNum == 0)
	nNum *= 10
	cTemp := STR(nNum)
	nNum := VAL("0."+SUBSTR(cTemp,AT(".",cTemp)+1))
	nRetVal++
END

RETURN(nRetVal)


/*-------------------------------------------------  FerrChk
* Syntax....FerrChk(xValue)
* Purpose...Break on low level file errors
* Returns...xValue
* Version...1.00  9101.29 / 14:54
* Notice....Copyright (C) 1991 by T. McConnell
*/
STATIC FUNCTION FerrChk(xValue)

IF FERROR() # 0
	BREAK
ENDIF

RETURN(xValue)


/*-------------------------------------------------  VarLoad
* Syntax....VarLoad(xNew,cFile)
* Purpose...Load a saved variable into 'xNew' from 'cFile'
* Returns...0 if it worked, FERROR() if it didn't
* Version...1.00  9101.29 / 15:12
* Notice....Copyright (C) 1991 by T. McConnell
*/
FUNCTION VarLoad(xNew,cFile)                     //  xNew should be passed by reference

LOCAL nRetVal := -1                              //  Assume unexpected EOF()
LOCAL nHan    := 0                               //  File handle to read from
LOCAL cIdStr  := SPACE(3)                        //  File ID string
LOCAL nLen    := 0                               //  Length of the surface array
LOCAL nEye    := 0                               //  Loop Counter
LOCAL cBuffer                                    //  Buffer for rest of stuff

BEGIN SEQUENCE                                   //  Used to trap FERROR()s
	IF !FILE(cFile)
		nRetVal := 2                               //  DOS's File not found
		BREAK                                      //  FERROR() will be 0. (0+2=2)
	ENDIF
	nHan    := FerrChk(FOPEN(cFile))
	ReadElem(@xNew,nHan)                          //  Read the variable
	nRetVal := FERROR()                           //  Clear unexpected EOF()

	FerrChk(FCLOSE(nHan))
END SEQUENCE

RETURN(IIF(FERROR()#0,FERROR(),nRetVal))


/*-------------------------------------------------  ReadElem
* Syntax....ReadElem(xNewElem)
* Purpose...Reads next element from file
* Returns...NIL
* Version...1.00  9101.29 / 16:41
* Notice....Copyright (C) 1991 by T. McConnell
*/
STATIC FUNCTION ReadElem(xNewElem,nHan)

LOCAL cBuffer 
LOCAL nLen    
LOCAL nEye    
LOCAL kludge                                     //  Used for recursion

cBuffer := ReadNext(1, nHan)
nLen    := 0
nEye    := 0

DO CASE
	CASE cBuffer == TYPENIL
		xNewElem := NIL

	CASE cBuffer == TYPEChar  
		cBuffer  := ReadNext(2, nHan)
		nLen     := BIN2W(cBuffer)

		cBuffer  := ReadNext(nLen, nHan)
		xNewElem := cBuffer

	CASE cBuffer == TYPETrue  
		xNewElem := true

	CASE cBuffer == TYPEFalse 
		xNewElem := false

	CASE cBuffer == TYPENum   
		cBuffer  := ReadNext(2, nHan)
		nLen     := ASC(LEFT(cBuffer,1)) + ASC(RIGHT(cBuffer,1)) + 1

		cBuffer  := ReadNext(nLen, nHan)
		xNewElem := VAL(cBuffer)
			
	CASE cBuffer == TYPEDate  
		cBuffer  := ReadNext(8, nHan)
		xNewElem := CTOD(TRANSFORM(RIGHT(cBuffer,4),"@R ##/##") + "/" + LEFT(cBuffer,4))

	CASE cBuffer == TYPEArray
		cBuffer  := ReadNext(2, nHan)
		xNewElem := {}
		ASIZE(xNewElem,BIN2W(cBuffer))

		//---
		// AEVAL(xNewElem, {|Elem| ReadElem(@Elem,nHan)})
      // ^ This won't work
      //--- But this is what that code does:
				
		FOR nLen := 1 TO LEN(xNewElem)
			ReadElem(@kludge,nHan)
			xNewElem[nLen] := kludge
		NEXT nLen

	OTHERWISE
		BREAK                                      //  File Corruption!
ENDCASE

RETURN(NIL)


/*-------------------------------------------------  ReadNext
* Syntax....ReadNext(nBytes, nHan)
* Purpose...Reads next nBytes from nHan
* Returns...String with next nBytes in it
* Version...1.00  9101.29 / 16:08  TGM
* Notice....Copyright (C) 1991 by T. McConnell
*/
STATIC FUNCTION ReadNext(nBytes, nHan)
LOCAL cBuffer := SPACE(nBytes)
LOCAL e

IF FerrChk(FREAD(nHan,@cBuffer,nBytes)) # nBytes
	BREAK                                         //  Unexpected EOF()
ENDIF

RETURN(cBuffer)
*--------------------------------------------------  VARFILES.PRG
