*=----------------------------------------------------------------------------
* Program...VFDEMO.PRG                                  Demo of VarFile calls
* Version...1.00  9101.30 / 10:18
*           1.10  9102.16 / 16:20 - Added error checking
*
* Notes.....Compile with /n, link with VARFILES.OBJ
* Notice....Copyright (C) 1991 by Terry McConnell
*=----------------------------------------------------------------------------

#translate true => .t.                           //  Just because it's easier
#translate false => .f.                          //  to type ...

*--------------------------------------------------  Multi-Dim Demo
FUNCTION Main()

LOCAL nErr                                       //  Return status
LOCAL a2Load  := {}                              //  Array to load into
LOCAL a2Save  := {"Hello",;                      //  Array to save from
                  "World",;
                  {},;
                  {1,2,3,4,"Five",6},;
                  DATE(),;
                  {{true,false},{1,0},{"Yes","No"}},;
                  "",;
                  {"There's","Nothing","In","Here",NIL,NIL,NIL},;
                  "End of the world as we know it."}

*--------------------------------------------------  What it is ...
? "VarFiles Demo, V1.1"
? "Copyright (C) 1991 by Terry McConnell"
?
? "Here's the array to save: " + Exp2Char(a2Save) //  Show the array
?
? "Saving ..."
IF (nErr := VarSave( a2Save,"DEMO.MDA")) # 0     //  Save to DEMO.MDA
   ? "DOS Error " + LTRIM(STR(nErr)) + " occurred"
ENDIF

? "Loading ..."
IF (nErr := VarLoad(@a2Load,"DEMO.MDA")) # 0     //  Note Pass by reference!
   IF nErr == -1                                 //  Not a DOS error ...
      ? ".MDA File is corrupted - unexpected end of file" 
   ELSE
      ? "DOS Error " + LTRIM(STR(nErr)) + " occurred"
   ENDIF
ENDIF	
?
? "Here's the one it loaded: " + Exp2Char(a2Load)

RETURN(NIL)


*--------------------------------------------------  Exp2Char
* Syntax....Exp2Char(xExp)
* Purpose...Translates xExp to character
*
* Version...1.00  9012.12 / TGM
* Notice....Copyright (C) 1991 by Terry McConnell
*
FUNCTION Exp2Char(xExp)

DO CASE
   CASE xExp == NIL
      RETURN("NIL")
   CASE VALTYPE(xExp) == "A"
      MakeStr("{")                               //  Warning!  Recursion.
      AEVAL(xExp,;
         {|xElement,nElement|;
            MakeStr(Exp2Char(xElement)+IIF(nElement==LEN(xExp),"",", "));
         };
      )
      MakeStr("}")
      RETURN(MakeStr())

   CASE VALTYPE(xExp) == "C"
      RETURN(CHR(34)+xExp+CHR(34))

   CASE VALTYPE(xExp) == "D"
      RETURN(DTOC(xExp))

   CASE VALTYPE(xExp) == "L"
      RETURN(IIF(xExp,"T","F"))

   CASE VALTYPE(xExp) == "N"
      RETURN(LTRIM(TRIM(STR(xExp))))

   CASE VALTYPE(xExp) == "O"                     //  Can't show Objects ...
      RETURN("Object")

   CASE VALTYPE(xExp) == "B"                     //  Can't show blocks ...
      RETURN("{||...}")
ENDCASE

RETURN("Exp2Char() Error ("+VALTYPE(xExp)+")")

*--------------------------------------------------  SUPPORT FUNCTION MakeStr
STATIC FUNCTION MakeStr(cMsg)
STATIC cRetStr
LOCAL cRetComplete

IF cMsg == NIL                                   //  Return built string, clear old one
   cRetComplete := cRetStr
   cRetStr      := NIL
   RETURN(cRetComplete)
ENDIF
   
IF cRetStr == NIL
   RETURN(cRetStr := cMsg)
ENDIF

RETURN(cRetStr += cMsg)
*--------------------------------------------------  EOF: VFDEMO.PRG
